/*
 * Permission is hereby granted, free of charge, to any person obtaining a 
 * copy of this software and associated documentation files (the "Software"), 
 * to deal in the Software without restriction, including without limitation 
 * the rights to use, copy, modify, merge, publish, distribute, sublicense, 
 * and/or sell copies of the Software, and to permit persons to whom the 
 * Software is furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in 
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR 
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, 
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL 
 * THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER 
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING 
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER 
 * DEALINGS IN THE SOFTWARE.
 */

package nu.validator.servlet;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.OutputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.util.Map;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.servlet.http.HttpSession;
import javax.servlet.http.Cookie;
import java.util.Enumeration;
import java.security.Principal;
import javax.servlet.RequestDispatcher;
import java.io.InputStream;
import javax.servlet.ServletInputStream;
import javax.servlet.ServletOutputStream;
import java.util.Locale;
import java.io.PrintWriter;

import org.apache.log4j.Logger;
import org.apache.log4j.Level;
import org.apache.log4j.BasicConfigurator;

class MockServletInputStream extends ServletInputStream {
    InputStream is;

    public MockServletInputStream(InputStream is) {
        this.is = is;
    }

    public int read() throws IOException {
        return this.is.read();
    }
}

class MockServletOutputStream extends ServletOutputStream {
    OutputStream os;

    public MockServletOutputStream(OutputStream os) {
        this.os = os;
    }

    public void write(int sz) throws IOException {
        this.os.write(sz);
    }
}

class MockHttpServletRequest implements HttpServletRequest {
    private int contentLength;

    private String filename;

    private String contentType;

    private boolean isStdIn;

    public void setInputFilename(String filename) {
        this.filename = filename;
    }

    public void setStdIn() {
        this.isStdIn = true;
    }

    public String getAuthType() {
        return null;
    }

    public String getContextPath() {
        return null;
    }

    public Cookie[] getCookies() {
        return null;
    }

    public long getDateHeader(String name) {
        return 0;
    }

    public String getHeader(String name) {
        return null;
    }

    public Enumeration getHeaderNames() {
        return null;
    }

    public Enumeration getHeaders(String name) {
        return null;
    }

    public int getIntHeader(String name) {
        return 0;
    }

    public String getMethod() {
        return "POST";
    }

    public String getPathInfo() {
        return null;
    }

    public String getPathTranslated() {
        return null;
    }

    public String getQueryString() {
        return null;
    }

    public String getRemoteUser() {
        return null;
    }

    public String getRequestedSessionId() {
        return null;
    }

    public String getRequestURI() {
        return null;
    }

    public StringBuffer getRequestURL() {
        return null;
    }

    public String getServletPath() {
        return null;
    }

    public HttpSession getSession() {
        return null;
    }

    public HttpSession getSession(boolean create) {
        return null;
    }

    public Principal getUserPrincipal() {
        return null;
    }

    public boolean isRequestedSessionIdFromCookie() {
        return true;
    }

    public boolean isRequestedSessionIdFromUrl() {
        return true;
    }

    public boolean isRequestedSessionIdFromURL() {
        return true;
    }

    public boolean isRequestedSessionIdValid() {
        return true;
    }

    public boolean isUserInRole(String role) {
        return true;
    }

    public Object getAttribute(String name) {
        return null;
    }

    public Enumeration getAttributeNames() {
        return null;
    }

    public String getCharacterEncoding() {
        return null;
    }

    public int getContentLength() {
        return contentLength;
    }

    public String getContentType() {
        this.contentType = System.getProperty(
                "nu.validator.client.content-type", null);
        if (this.filename.endsWith(".xhtml")) {
            this.contentType = this.contentType == null ? "application/xhtml+xml"
                    : this.contentType;
        } else {
            this.contentType = this.contentType == null ? "text/html"
                    : this.contentType;
        }
        return this.contentType;
    }

    public ServletInputStream getInputStream()
            throws java.io.FileNotFoundException {
        File f = new File(filename);
        contentLength = (int) f.length();
        InputStream is = new FileInputStream(f);
        return new MockServletInputStream(is);
    }

    public String getLocalAddr() {
        return null;
    }

    public Locale getLocale() {
        return null;
    }

    public Enumeration getLocales() {
        return null;
    }

    public String getLocalName() {
        return null;
    }

    public int getLocalPort() {
        return 80;
    }

    public String getParameter(String name) {
        if ("doc".equals(name)) {
            return this.isStdIn ? "[stdin]" : this.filename;
        }
        if ("out".equals(name)) {
            return System.getProperty("nu.validator.client.out", "gnu");
        }
        if ("level".equals(name)) {
            return System.getProperty("nu.validator.client.level", null);
        }
        if ("parser".equals(name)) {
            String parser = System.getProperty("nu.validator.client.parser",
                    null);
            if ("application/xhtml+xml".equals(contentType)) {
                return parser == null ? "xml" : parser;
            } else {
                return parser;
            }
        }
        if ("charset".equals(name)) {
            return System.getProperty("nu.validator.client.charset", null);
        }
        if ("laxtype".equals(name)) {
            return System.getProperty("nu.validator.client.laxtype", "yes");
        }
        if ("asciiquotes".equals(name)) {
            if (!"no".equals(System.getProperty("nu.validator.client.asciiquotes"))) {
                return "yes";
            }
        }
        return null;
    }

    public Map getParameterMap() {
        return null;
    }

    public Enumeration getParameterNames() {
        return null;
    }

    public String[] getParameterValues(String name) {
        return null;
    }

    public String getProtocol() {
        return null;
    }

    public BufferedReader getReader() {
        return null;
    }

    public String getRealPath(String path) {
        return null;
    }

    public String getRemoteAddr() {
        return null;
    }

    public String getRemoteHost() {
        return null;
    }

    public int getRemotePort() {
        return 9999;
    }

    public RequestDispatcher getRequestDispatcher(String path) {
        return null;
    }

    public String getScheme() {
        return null;
    }

    public String getServerName() {
        return null;
    }

    public int getServerPort() {
        return 80;
    }

    public boolean isSecure() {
        return false;
    }

    public void removeAttribute(String name) {
    }

    public void setAttribute(String name, Object o) {
    }

    public void setCharacterEncoding(String env) {
    }
}

class MockHttpServletResponse implements HttpServletResponse {
    private static final Logger log4j = Logger.getLogger(MockHttpServletResponse.class);

    public void addCookie(Cookie cookie) {
    }

    public void addDateHeader(String name, long date) {
    }

    public void addHeader(String name, String value) {
    }

    public void addIntHeader(String name, int value) {
    }

    public boolean containsHeader(String name) {
        return false;
    }

    public String encodeRedirectUrl(String url) {
        return null;
    }

    public String encodeRedirectURL(String url) {
        return null;
    }

    public String encodeUrl(String url) {
        return null;
    }

    public String encodeURL(String url) {
        return null;
    }

    public void sendError(int sc) {
    }

    public void sendError(int sc, String msg) {
        log4j.error(msg);
    }

    public void sendRedirect(String location) {
    }

    public void setDateHeader(String name, long date) {
    }

    public void setHeader(String name, String value) {
    }

    public void setIntHeader(String name, int value) {
    }

    public void setStatus(int sc) {
    }

    public void setStatus(int sc, String sm) {
    }

    public void flushBuffer() {
    }

    public int getBufferSize() {
        return 1024;
    }

    public String getCharacterEncoding() {
        return null;
    }

    public String getContentType() {
        return null;
    }

    public Locale getLocale() {
        return null;
    }

    public ServletOutputStream getOutputStream() {
        return new MockServletOutputStream(System.out);
    }

    public PrintWriter getWriter() {
        return null;
    }

    public boolean isCommitted() {
        return false;
    }

    public void reset() {
    }

    public void resetBuffer() {
    }

    public void setBufferSize(int size) {
    }

    public void setCharacterEncoding(String charset) {
    }

    public void setContentLength(int len) {
    }

    public void setContentType(String type) {
    }

    public void setLocale(Locale loc) {
    }
}

class VerifierCommandLine {

    public static void main(String[] args) throws Exception {
        String filename = null;
        BasicConfigurator.configure();
        Logger root = Logger.getRootLogger();
        root.setLevel(Level.WARN);
        System.setProperty("org.whattf.datatype.warn", "1");
        System.setProperty("nu.validator.servlet.suppress-spec-load", "1");
        msgToStdErr("Initializing command-line validator...");
        msgToStdErr("To start the validator as a service instead, run \"java -cp vnu.jar nu.validator.servlet.Main 8888\".");
        msgToStdErr("For faster validation, run \"java -jar vnu-fast-client.jar FILE1.html FILE2.html...\".");
        msgToStdErr("For more information, see https://bitbucket.org/sideshowbarker/vnu/overview#readme");
        msgToStdErr("To quiet these messages, run with \"java -Dnu.validator.client.quiet=yes ...\".");
        new VerifierServletTransaction(null, null);
        msgToStdErr("Initialization completed.");
        MockHttpServletRequest request = new MockHttpServletRequest();
        HttpServletResponse response = new MockHttpServletResponse();
        int filecount = (args.length == 0 ? 1 : args.length);
        for (int i = 0; i < filecount; i++) {
            if (args.length == 0 || "-".equals(args[i])) {
                msgToStdErr("Waiting for document content on standard input...");
                msgToStdErr("(To validate files instead, run \"java -jar vnu.jar FILE1.html FILE2.html...\")");
                request.setStdIn();
                File doc = File.createTempFile("tmp", ".html");
                doc.deleteOnExit();
                OutputStream os = new FileOutputStream(doc);
                int b;
                while ((b = System.in.read()) != -1) {
                    os.write((byte) b);
                }
                os.close();
                filename = doc.getAbsolutePath();
            } else {
                filename = args[i];
            }
            request.setInputFilename(filename);
            new VerifierServletTransaction(request, response).service();
        }
    }

    private static final void msgToStdErr(String msg) {
        if (!"yes".equals(System.getProperty("nu.validator.client.quiet", "no"))) {
            System.err.println(msg);
        }
    }
}
